@@ -20,11 +20,15 @@ import (
 	"bytes"
 	"errors"
 	"io"
+	"io/fs"
 	"io/ioutil"
 	"os"
 	"path/filepath"
 	"strings"
 
+	goast "go/ast"
+	goparser "go/parser"
+
 	"github.com/goplus/gop/ast"
 	"github.com/goplus/gop/token"
 )
@@ -49,19 +53,19 @@ func SetDebug(dbgFlags int) {
 
 // FileSystem represents a file system.
 type FileSystem interface {
-	ReadDir(dirname string) ([]os.FileInfo, error)
+	ReadDir(dirname string) ([]fs.FileInfo, error)
 	ReadFile(filename string) ([]byte, error)
 	Join(elem ...string) string
 }
 
 type localFS struct{}
 
-func (p localFS) ReadDir(dirname string) ([]os.FileInfo, error) {
+func (p localFS) ReadDir(dirname string) ([]fs.FileInfo, error) {
 	return ioutil.ReadDir(dirname)
 }
 
 func (p localFS) ReadFile(filename string) ([]byte, error) {
-	return ioutil.ReadFile(filename)
+	return os.ReadFile(filename)
 }
 
 func (p localFS) Join(elem ...string) string {
@@ -96,15 +100,27 @@ func astFileToPkg(file *ast.File, fileName string) (pkg *ast.Package) {
 
 // ParseDir calls ParseFSDir by passing a local filesystem.
 //
-func ParseDir(fset *token.FileSet, path string, filter func(os.FileInfo) bool, mode Mode) (pkgs map[string]*ast.Package, first error) {
-	return ParseFSDir(fset, local, path, filter, mode)
+func ParseDir(fset *token.FileSet, path string, filter func(fs.FileInfo) bool, mode Mode) (pkgs map[string]*ast.Package, first error) {
+	return ParseFSDir(fset, local, path, Config{Filter: filter, Mode: mode})
+}
+
+type Config struct {
+	IsClass func(ext string) (isProj bool, ok bool)
+	Filter  func(fs.FileInfo) bool
+	Mode    Mode
+}
+
+// ParseDirEx calls ParseFSDir by passing a local filesystem.
+//
+func ParseDirEx(fset *token.FileSet, path string, conf Config) (pkgs map[string]*ast.Package, first error) {
+	return ParseFSDir(fset, local, path, conf)
 }
 
 // ParseFSDir calls ParseFile for all files with names ending in ".gop" in the
 // directory specified by path and returns a map of package name -> package
 // AST with all the packages found.
 //
-// If filter != nil, only the files with os.FileInfo entries passing through
+// If filter != nil, only the files with fs.FileInfo entries passing through
 // the filter (and ending in ".gop") are considered. The mode bits are passed
 // to ParseFile unchanged. Position information is recorded in fset, which
 // must not be nil.
@@ -113,11 +129,14 @@ func ParseDir(fset *token.FileSet, path string, filter func(os.FileInfo) bool, m
 // returned. If a parse error occurred, a non-nil but incomplete map and the
 // first error encountered are returned.
 //
-func ParseFSDir(fset *token.FileSet, fs FileSystem, path string, filter func(os.FileInfo) bool, mode Mode) (pkgs map[string]*ast.Package, first error) {
+func ParseFSDir(fset *token.FileSet, fs FileSystem, path string, conf Config) (pkgs map[string]*ast.Package, first error) {
 	list, err := fs.ReadDir(path)
 	if err != nil {
 		return nil, err
 	}
+	if conf.IsClass == nil {
+		conf.IsClass = defaultIsClass
+	}
 	pkgs = make(map[string]*ast.Package)
 	for _, d := range list {
 		if d.IsDir() {
@@ -125,27 +144,39 @@ func ParseFSDir(fset *token.FileSet, fs FileSystem, path string, filter func(os.
 		}
 		fname := d.Name()
 		ext := filepath.Ext(fname)
-		ft, isOk := extGopFiles[ext]
-		if ft == ast.FileTypeGo && (mode&ParseGoFiles) == 0 {
-			isOk = false
+		var isProj, isClass, useGoParser bool
+		switch ext {
+		case ".gop":
+		case ".go":
+			if strings.HasPrefix(fname, "gop_autogen") {
+				continue
+			}
+			useGoParser = (conf.Mode & ParseGoAsGoPlus) == 0
+		default:
+			if isProj, isClass = conf.IsClass(ext); !isClass {
+				continue
+			}
 		}
-		if isOk && !strings.HasPrefix(fname, "_") && (filter == nil || filter(d)) {
+		if !strings.HasPrefix(fname, "_") && (conf.Filter == nil || conf.Filter(d)) {
 			filename := fs.Join(path, fname)
-			if filedata, err := fs.ReadFile(filename); err == nil {
-				if src, err := ParseFSFile(fset, fs, filename, filedata, mode); err == nil {
-					name := src.Name.Name
-					pkg, found := pkgs[name]
-					if !found {
-						pkg = &ast.Package{
-							Name:  name,
-							Files: make(map[string]*ast.File),
+			if useGoParser {
+				if filedata, err := fs.ReadFile(filename); err == nil {
+					if src, err := goparser.ParseFile(fset, filename, filedata, goparser.Mode(conf.Mode)); err == nil {
+						pkg := reqPkg(pkgs, src.Name.Name)
+						if pkg.GoFiles == nil {
+							pkg.GoFiles = make(map[string]*goast.File)
 						}
-						pkgs[name] = pkg
+						pkg.GoFiles[filename] = src
+					} else {
+						first = err
 					}
-					pkg.Files[filename] = src
-				} else if first == nil {
+				} else {
 					first = err
 				}
+			} else if src, err := ParseFSFile(fset, fs, filename, nil, conf.Mode); err == nil {
+				src.IsProj, src.IsClass = isProj, isClass
+				pkg := reqPkg(pkgs, src.Name.Name)
+				pkg.Files[filename] = src
 			} else if first == nil {
 				first = err
 			}
@@ -154,24 +185,27 @@ func ParseFSDir(fset *token.FileSet, fs FileSystem, path string, filter func(os.
 	return
 }
 
-var (
-	extGopFiles = map[string]ast.FileType{
-		".go":  ast.FileTypeGo,
-		".gop": ast.FileTypeGop,
-		".spx": ast.FileTypeSpx,
-		".gmx": ast.FileTypeGmx,
+func reqPkg(pkgs map[string]*ast.Package, name string) *ast.Package {
+	pkg, found := pkgs[name]
+	if !found {
+		pkg = &ast.Package{
+			Name:  name,
+			Files: make(map[string]*ast.File),
+		}
+		pkgs[name] = pkg
 	}
-)
+	return pkg
+}
 
-// RegisterFileType registers a new Go+ class file type.
-func RegisterFileType(ext string, format ast.FileType) {
-	if format != ast.FileTypeSpx && format != ast.FileTypeGmx {
-		panic("RegisterFileType: format should be FileTypeSpx or FileTypeGmx")
+func defaultIsClass(ext string) (isProj bool, ok bool) {
+	switch ext {
+	case ".gmx":
+		isProj = true
+		fallthrough
+	case ".spx":
+		ok = true
 	}
-	if _, ok := extGopFiles[ext]; ok {
-		panic("RegisterFileType: file type exists")
-	}
-	extGopFiles[ext] = format
+	return
 }
 
 // -----------------------------------------------------------------------------
@@ -207,11 +241,6 @@ func ParseFile(fset *token.FileSet, filename string, src interface{}, mode Mode)
 
 // ParseFSFile parses the source code of a single Go+ source file and returns the corresponding ast.File node.
 func ParseFSFile(fset *token.FileSet, fs FileSystem, filename string, src interface{}, mode Mode) (f *ast.File, err error) {
-	ext := filepath.Ext(filename)
-	ft, isOk := extGopFiles[ext]
-	if !isOk {
-		ft = ast.FileTypeGop
-	}
 	var code []byte
 	if src == nil {
 		code, err = fs.ReadFile(filename)
@@ -221,11 +250,7 @@ func ParseFSFile(fset *token.FileSet, fs FileSystem, filename string, src interf
 	if err != nil {
 		return
 	}
-	f, err = parseFile(fset, filename, code, mode)
-	if f != nil {
-		f.FileType = ft
-	}
-	return
+	return parseFile(fset, filename, code, mode)
 }
 
 var (
